try:
    from Products.validation.interfaces.IValidator import IValidator
    from Acquisition import aq_base
except ImportError:
    import sys, os
    sys.path.insert(0, os.path.join(os.path.dirname(__file__), os.pardir))
    from interfaces.IValidator import IValidator
    del sys, os

_marker = []

try:
    True
except NameError:
    True=1
    False=0

from types import FileType
try:
    from ZPublisher.HTTPRequest import FileUpload
except ImportError:
    FileUpload = FileType

class MaxSizeValidator:
    __implements__ = IValidator

    def __init__(self, name, title='', description='', maxsize=0):
        self.name = name
        self.title = title or name
        self.description = description
        self.maxsize= maxsize

    def __call__(self, value, *args, **kwargs):
        instance = kwargs.get('instance', None)
        field    = kwargs.get('field', None)

        # get max size
        if kwargs.has_key('maxsize'):
            maxsize = kwargs.get('maxsize')
        elif hasattr(aq_base(instance), 'getMaxSizeFor'):
            maxsize = instance.getMaxSizeFor(field.getName())
        elif hasattr(field, 'maxsize'):
            maxsize = field.max_size
        else:
            maxsize = self.maxsize
        
        if not maxsize:
            return True
        
        # calculate size
        if hasattr(value, '__len__'):
            size = len(value)
        elif type(value) is FileType or isinstance(value, FileUpload):
            size = len(value.read())
            value.seek(0)
        else:
            raise Exception # XXX
        size = float(size)

        if (size * 1024 * 1024) > maxsize:
            return ("Validation failed(%(name)s: Uploaded data is too large: %(size).3fMB (max %(max)dMB)" %
                     { 'name' : self.name, 'size' : size / (1024 * 1024), 'max' : maxsize })
        else:
            return True


try:
    from DateTime import DateTime
except ImportError:
    # XXX use python datetime?
    DateTime = lambda date: None

class DateValidator:

    __implements__ = IValidator

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        if not value:
            return ("Validation failed(%s): value is "
                    "empty (%s)." % (self.name, repr(value)))
        if not isinstance(value, DateTime):
            try:
                value = DateTime(value)
            except:
                return ("Validation failed(%s): could not "
                        "convert %s to a date.""" % (self.name, value))
        return True


validatorList = [
    MaxSizeValidator('isMaxSize', title='', description=''),
    DateValidator('isValidDate', title='', description=''),
    ]

__all__ = ('validatorList', )

